#include <complex>
#include "memalloc.h"
#include "pde.h"
#define pi 3.141592653589793

using namespace std;
typedef complex<double> dcmplx;                        

double Pot(double x, double a, double V0)                        // Potencjał
{
    return fabs(x) <= 0.5e0 * a ? V0 : 0e0;
}
void Init(dcmplx Psi[], double x[], int nx, double x0, double sig, double k0)
{
    double a, b, dx, f;
    int i;

    a = 1e0 / sqrt(sqrt(2e0 * pi) * sig);
    b = -1e0 / (4e0 * sig * sig);
    for (i = 1; i <= nx; i++) {
        dx = x[i] - x0;
        f = a * exp(b * dx * dx); if (f < 1e-10) f = 0e0;
        Psi[i] = f * dcmplx(cos(k0 * x[i]), sin(k0 * x[i]));
    }
}
void ProbDens(dcmplx Psi[], double Psi2[], int nx, double hx, double& PsiNorm)
//Obliczanie gęstości prawdopodobieństwa Psi2[] funkcji falowej Psi[]
{
    int i;

    for (i = 1; i <= nx; i++) {
        Psi2[i] = norm(Psi[i]);             // nieznormalizowana gęstość prawdopodobieństwa
        if (Psi2[i] < 1e-10) Psi2[i] = 0e0;
    }

    PsiNorm = 0.5e0 * (Psi2[1] + Psi2[nx]);      // całka za pomocą trapezu
    for (i = 2; i <= (nx - 1); i++) PsiNorm += Psi2[i];
    PsiNorm *= hx;

    for (i = 1; i <= nx; i++) Psi2[i] /= PsiNorm;      // znormalizowane prawd. gęstość
}
int main()
{
    dcmplx* Psi;
    double* Psi2, * V, * x;
    double a, sig, ht, hx, k0, PsiNorm, t, tmax, V0, x0, xmax;
    int i, it, nout, nt, nx, nx2;
    char fname[80];
    FILE* out;

    a = 5e0;                                  // szerokość potencjalnej bariery
    V0 = 50e0;                                // wysokość potencjalnej bariery
    x0 = -20e0;                           // początkowa pozycja fali
    sig = 1e0;                                        // połowa szerokości pakietu
    k0 = 10e0;                              // średnia liczba fali pakietu
    xmax = 100e0;                                                 // max x
    hx = 5e-2;                                          // rozmiar kroku
    tmax = 5e0;                                    // maksymalny czas propagacji
    ht = 5e-3;                                                  // krok czasowy
    nout = 40;                                      

    nx = 2 * (int)(xmax / hx + 0.5) + 1;            // nieparzysta liczba węzłów przestrzennych
    nt = (int)(tmax / ht + 0.5);                         // liczba kroków w czasie
    nx2 = nx / 2;

    Psi = CVector(1, nx);                                     // funkcja fali
    Psi2 = Vector(1, nx);                                // gęstość prawdopodobieństwa
    V = Vector(1, nx);                                             // potencjał
    x = Vector(1, nx);                                    // współrzędna przestrzenna

    for (i = 1; i <= nx; i++) {             
        x[i] = (i - nx2 - 1) * hx;
        V[i] = Pot(x[i], a, V0);
    }

    Init(Psi, x, nx, x0, sig, k0);                           

    for (it = 1; it <= nt; it++) {                                    // pętla czasowa
        t = it * ht;
        PropagQTD(Psi, V, nx, hx, ht);  

        ProbDens(Psi, Psi2, nx, hx, PsiNorm);               // gęstość prawdopodobieństwa

        if (it % nout == 0 || it == nt) {            
            sprintf(fname, "scatter_%4.2f.txt", t);
            out = fopen(fname, "w");
            fprintf(out, "t = %4.2f\n", t);
            fprintf(out, "     x          V        PsiR      PsiI      Psi2\n");
            for (i = 1; i <= nx; i++)
                fprintf(out, "%10.5f%10.5f%10.5f%10.5f%10.5f\n",
                    x[i], V[i], real(Psi[i]), imag(Psi[i]), Psi2[i]);
            fclose(out);
        }
    }
}